/*
 *  docbook-to-latex using dancer-XML parser
 *  Copyright (C) 2003 Junichi Uekawa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "dancer-xml.h"

/* remove exceeding space, and remove special-chars */
static int convert_print (const char * s)
{
  if (!s)
    {
      return 1;
    }
  
  while (*s && isspace(*s))
    {
      s++;
    }
  while (*s)
    {
      switch (*s) 
	{
	case '$':
	  printf ("\\$");
	  break;
	case '_':
	  printf ("\\_");
	  break;
	case '\\':
	  printf ("\\\\_");
	  break;
	case '~':
	  printf ("\\~\\ ");
	  break;
	case '&':		/* TODO -- handle this */
	  break;
	default:
	  putc(*s,stdout);
	}
      
      s++;
      
    }
  
  return 0;
}


static const char * getatrstring(dxml_element * e, const char * attr)
{
  dxml_attribute * a = e->element_attribute;
  while (a)
    {
      if (!strcmp(a->attribute_name,attr))
	return a->attribute_data;
      a=a->next;
    }
  return NULL;
}


static int get_bookchapter(dxml_element *b, int paraenter);


static int get_itemizedlist(dxml_element *b)
{
  printf("\\begin{itemize}\n");
  while (b)
    {
      
      if (!strcasecmp("listitem", b->element_name))
	{
	  printf("\\item ");
	  get_bookchapter(b->child,0);
	  printf("\n");
	}
      else
	dxml_dump_element(b);
      
      b=b->next;
    }
  printf("\\end{itemize}\n");
  return 0;
}

/* handle para-type ones which can contain PCDATA */
static int get_paratype(dxml_element *b)
{
  while (b)
    {
      if (b->element_type == element_type_pcdata)
	{
	  convert_print (b->element_name);
	}
      else if (!strcasecmp("screen", b->element_name))
	{
	  puts("\\begin{verbatim}");
	  puts(b->child->element_name);
	  puts("\\end{verbatim}");
	}
      else if ((!strcasecmp("command", b->element_name)) ||
	       (!strcasecmp("filename", b->element_name))
	       )
	{
	  printf ("{\\tt ");
	  get_paratype(b->child);
	  printf ("} ");
	}
      else if ((!strcasecmp("option", b->element_name)) ||
	       (!strcasecmp("parameter", b->element_name)))
	{
	  printf ("{\\it ");
	  get_paratype(b->child);
	  printf ("} ");
	}
      else if (!strcasecmp("footnote", b->element_name))
	{
	  printf ("\\footnote{");
	  get_bookchapter(b->child,0);
	  printf ("} ");
	}
      else if (!strcasecmp("itemizedlist", b->element_name))
	{
	  get_itemizedlist(b->child);
	}
      else if (!strcasecmp("xref", b->element_name))
	{
	  printf ("\\ref{%s}", 
		  getatrstring(b, "linkend"));
	  get_paratype(b->child);
	}
      else if (!strcasecmp("ulink", b->element_name))
	{
	  get_paratype(b->child);
	  printf ("(\\verb!%s!)", 
		  getatrstring(b, "url"));	  
	}
      else
	{
	  fprintf(stderr, "Unknown tag: %s\n", b->element_name);
	  dxml_dump_element(b);
	}
      
      
      b=b->next;
    }
  return 0;
}



/* handle chapter and sect* */
static int get_bookchapter(dxml_element *b, int paraenter)
{
  while (b)
    {
      if (b->element_type != element_type_element)
	{
	  exit (1);
	}
      else if (!strcasecmp("sect1", b->element_name))
	{
	  printf ("\n\\section{");
	  convert_print(dxml_get_PCDATA_bysimplepath(b->child, "title"));
	  printf("}\n\n");
	  get_bookchapter(b->child,1);
	}
      else if (!strcasecmp("sect2", b->element_name))
	{
	  printf ("\n\\subsection{");
	  convert_print(dxml_get_PCDATA_bysimplepath(b->child, "title"));
	  printf("}\n\n");
	  get_bookchapter(b->child,1);
	}
      else if (!strcasecmp("sect3", b->element_name))
	{
	  printf ("\n\\subsubsection{");
	  convert_print(dxml_get_PCDATA_bysimplepath(b->child, "title"));
	  printf("}\n\n");
	  get_bookchapter(b->child,1);
	}
      else if (!strcasecmp("title", b->element_name))
	{
	}
      else if (!strcasecmp("screen", b->element_name))
	{
	  puts("\\begin{verbatim}");
	  puts(b->child->element_name);
	  puts("\\end{verbatim}");
	}
      else if (!strcasecmp("para", b->element_name))
	{
	  if (paraenter)puts("\n\n");
	  get_paratype(b->child);
	  if (paraenter)puts("\n\n");
	}
      else if (!strcasecmp("itemizedlist", b->element_name))
	{
	  get_itemizedlist(b->child);
	}
      else
	{
	  fprintf(stderr, "Unknown tag: %s\n", b->element_name);
	  dxml_dump_element(b->child);
	}
      
      b=b->next;
    }
  return 0;
}

static int recursebook(dxml_element * b)
{
  dxml_element * tmp;
  for (tmp = dxml_get_element_byname(b, "chapter"); tmp; tmp=tmp->next)
    {
      printf ("\n\\chapter{");
      convert_print(dxml_get_PCDATA_bysimplepath(tmp->child, "title"));
      printf("}\n\n");
      get_bookchapter (tmp->child,1);
    }
  return 0;
}


int
main(int argc, char ** argv)
{
  dxml_element * tmp ;
  
  /* printf("<?xml version=\"1.0\"?>\n"); */
  printf ("\\documentclass[a4j]{jreport}\n"
	  "\\usepackage{moreverb}\n"
	  "\\author{Junichi Uekawa}\n"
	  "\\date{}\n");

  tmp = dxml_read_xml(stdin);
  if (tmp)
    {
      printf (
	      "\\title{");
      convert_print(dxml_get_PCDATA_bysimplepath(tmp,"book/title"));
      printf (
	      "}\n"
	      "\\begin{document}\n"
	      "\\maketitle\n");
      
      recursebook(dxml_get_element_byname(tmp, "book")->child);
      printf (
	      "\\end{document}");
    }
  else
    {
      printf ("Something wicked happened!\n");
      return 1;
    }

  return 0;  
}
